//
//                 AVRMP3Car - Standalone MP3-Player
//                          Version 2.00 - experimental!   
// written 1999-2000 by Pototschnig Thomas (skymab@gmx.de)
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation - version 2.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License in file COPYING for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//Please report any bug/fix, modification, suggestion to
//
//           skymab@gmx.de
//
//--------------------------------------------------------------------
#include <io.h>
#include <progmem.h>
#include "global.h"
#include "sram.h"
#include "lcd240.h"
#include "port.h"
#include "button.h"


unsigned int VAR_TextHomeAdr = 0x0000;
unsigned int VAR_GraphHomeAdr= 0x0000;
unsigned int VAR_ADP         = 0x0000;
extern unsigned char ButtonTable [14][82];
extern unsigned char VAR_ButtonNumber [8];
extern unsigned char VAR_ButtonAttr [8];

/*-------------------------------------------------------------------
  LCD240_DisplayBar
-------------------------------------------------------------------*/
void LCD240_DisplayBar (unsigned char YPOS)
{

unsigned int WorkCounter = 0x0000;

    LCD240_SetADP(VAR_GraphHomeAdr+(YPOS*320));
    for (WorkCounter=0x0000; WorkCounter<320; WorkCounter+=1)
        LCD240_OutChar(0x3f);
}

/*-------------------------------------------------------------------
  LCD240_ClearBar
-------------------------------------------------------------------*/
void LCD240_ClearBar (unsigned char YPOS)
{

unsigned int WorkCounter = 0x0000;

    LCD240_SetADP(VAR_GraphHomeAdr+(YPOS*320));
    for (WorkCounter=0x0000; WorkCounter<320; WorkCounter+=1)
        LCD240_OutChar(0x00);
}

/*-------------------------------------------------------------------
  LCD240_WaitForReady
-------------------------------------------------------------------*/
void LCD240_WaitForReady (void)
{

unsigned char WorkByte   = 0;

    WorkByte = 0;

    SRAM_Disable ();                    // speicherverwaltung deaktiv
    PORT_SetPortAInput ();              // Und Port auf Input schalten
    sbi (PORTD, LCD_CD);                // Data addressieren
    cbi (PORTE, LCD_CS);                // Display aktivieren

    while ((WorkByte & 0x03) != 0x03)
    {
        cbi (PORTE, PER_RD);            // Leseleitung aktivieren
        WorkByte = inp (PINA);          // Byte einlesen
        sbi (PORTE, PER_RD);            // und wieder deaktivieren
    }

    sbi (PORTE, LCD_CS);                // CS-deaktivieren
    PORT_SetPortAOutput ();            // PortA als Output
    SRAM_Enable ();                     // Speicherverwaltung wieder ein

}

/*-------------------------------------------------------------------
  LCD240_WriteByte
-------------------------------------------------------------------*/
void LCD240_WriteByte (unsigned char WorkByte)
{
    SRAM_Disable ();                    // Speicherverwaltung aus
    cbi (PORTE, LCD_CS);                // LCD-Display aktivieren
    outp (WorkByte, PORTA);             // Byte rausschreiben auf PortA
    cbi (PORTE, PER_WR);                // Write-Leitung stroben
    sbi (PORTE, PER_WR);
    sbi (PORTE, LCD_CS);                // LCD-Display deaktivieren
    SRAM_Enable ();                     // Speicherverwaltung ein
}

/*-------------------------------------------------------------------
  LCD240_WriteData
-------------------------------------------------------------------*/
void LCD240_WriteData (unsigned char WorkByte)
{

    LCD240_WaitForReady ();             // Warten auf Display
    cbi (PORTD, LCD_CD);                // Data addressieren
    LCD240_WriteByte (WorkByte);        // Byte rausschreiben

}

/*-------------------------------------------------------------------
 LCD240_CrsrPatSel
-------------------------------------------------------------------*/
void LCD240_CrsrPatSel (unsigned char WorkByte)
{
    WorkByte &= 0xa0;
    LCD240_WriteCommand (WorkByte);
}

/*-------------------------------------------------------------------
  LCD240_WriteCommand
-------------------------------------------------------------------*/
void LCD240_WriteCommand (unsigned char WorkByte)
{

    LCD240_WaitForReady ();             // Warten auf Display
    sbi (PORTD, LCD_CD);                // Data addressieren
    LCD240_WriteByte (WorkByte);        // Byte rausschreiben

}

/*-------------------------------------------------------------------
 LCD240_TextHomeAdr
-------------------------------------------------------------------*/
void LCD240_SetTextHomeAdr (unsigned int WorkAdr)
{

    VAR_TextHomeAdr = WorkAdr;
    LCD240_WriteData (lo(WorkAdr));
    LCD240_WriteData (hi(WorkAdr));
    LCD240_WriteCommand (0x40);
}

/*-------------------------------------------------------------------
 LCD240_GraphHomeAdr
-------------------------------------------------------------------*/
void LCD240_SetGraphHomeAdr (unsigned int WorkAdr)
{

    VAR_GraphHomeAdr = WorkAdr;
    LCD240_WriteData (lo(WorkAdr));
    LCD240_WriteData (hi(WorkAdr));
    LCD240_WriteCommand (0x42);
}


/*-------------------------------------------------------------------
 LCD240_GetGraphHomeAdr
-------------------------------------------------------------------*/
unsigned int LCD240_GetGraphHomeAdr (void)
{
    return VAR_GraphHomeAdr;
}

/*-------------------------------------------------------------------
 LCD240_GetTextHomeAdr
-------------------------------------------------------------------*/
unsigned int LCD240_GetTextHomeAdr (void)
{
    return VAR_TextHomeAdr;
}

/*-------------------------------------------------------------------
 LCD240_SetADP
-------------------------------------------------------------------*/
void LCD240_SetADP (unsigned int WorkAdr)
{
    VAR_ADP = WorkAdr;
    LCD240_WriteData (lo(WorkAdr));
    LCD240_WriteData (hi(WorkAdr));
    LCD240_WriteCommand (0x24);
}

/*-------------------------------------------------------------------
 LCD240_GetADP                  ADP in Y!
-------------------------------------------------------------------*/
unsigned int LCD240_GetADP (void)
{
    return VAR_ADP;
}

/*-------------------------------------------------------------------
 LCD240_OfsRegSet
-------------------------------------------------------------------*/
void LCD240_OfsRegSet (unsigned char WorkByte)
{
    LCD240_WriteData(WorkByte);
    LCD240_WriteData(0x00);
    LCD240_WriteCommand(0x22);
}

/*-------------------------------------------------------------------
 LCD240_TextArea
-------------------------------------------------------------------*/
void LCD240_TextArea(unsigned char WorkByte)
{
    LCD240_WriteData(WorkByte);
    LCD240_WriteData(0x00);
    LCD240_WriteCommand(0x41);
}

/*-------------------------------------------------------------------
 LCD240_GraphArea
-------------------------------------------------------------------*/
void LCD240_GraphArea(unsigned char WorkByte)
{
    LCD240_WriteData(WorkByte);
    LCD240_WriteData(0x00);
    LCD240_WriteCommand(0x43);
}

/*-------------------------------------------------------------------
 LCD240_DisplayMode
-------------------------------------------------------------------*/
void LCD240_DisplayMode(unsigned char WorkByte)
{
    LCD240_WriteCommand(WorkByte | 0x90);
}

/*-------------------------------------------------------------------
 LCD240_AutoModeOn
-------------------------------------------------------------------*/
void LCD240_AutoModeOn(void)
{
    LCD240_WriteCommand(0xb0);
}

/*-------------------------------------------------------------------
 LCD240_AutoModeOff
-------------------------------------------------------------------*/
void LCD240_AutoModeOff(void)
{
    LCD240_WriteCommand(0xb2);
}

/*-------------------------------------------------------------------
 LCD240_CombineMode
-------------------------------------------------------------------*/
void LCD240_CombineMode(unsigned char WorkByte)
{
    LCD240_WriteCommand(WorkByte | 0x80);
}

/*-------------------------------------------------------------------
 LCD240_SetCursor
-------------------------------------------------------------------*/
void LCD240_SetCursor(unsigned char WorkByteX,
                      unsigned char WorkByteY)
{

unsigned int WorkAdr;

    LCD240_WriteData(WorkByteX);
    LCD240_WriteData(WorkByteY);
    LCD240_WriteCommand(0x21);

    WorkAdr = LCD240_GetTextHomeAdr();
    WorkAdr = WorkAdr + (WorkByteY * 40) + WorkByteX;
    LCD240_SetADP (WorkAdr);
}

/*-------------------------------------------------------------------
 LCD240_ClearText
-------------------------------------------------------------------*/
void LCD240_ClearText(void)
{

unsigned int WorkCounter = 0;

    LCD240_SetADP (LCD240_GetTextHomeAdr());
    LCD240_AutoModeOn();

    for (WorkCounter=0; WorkCounter< 320; WorkCounter=WorkCounter+1)
    {
        LCD240_WriteData(0x00);
    }

    LCD240_AutoModeOff();
    LCD240_SetADP (LCD240_GetTextHomeAdr());
}

/*-------------------------------------------------------------------
 LCD240_ClearGraph
-------------------------------------------------------------------*/
void LCD240_ClearGraph(void)
{

unsigned int WorkCounter = 0;

    LCD240_SetADP (LCD240_GetGraphHomeAdr());
    LCD240_AutoModeOn();

    for (WorkCounter=0; WorkCounter< (40*64); WorkCounter=WorkCounter+1)
    {
        LCD240_WriteData(0x00);
    }

    LCD240_AutoModeOff();
    LCD240_SetADP (LCD240_GetTextHomeAdr());
}

/*-------------------------------------------------------------------
 LCD240_OutChar
-------------------------------------------------------------------*/
void LCD240_OutChar(unsigned char WorkByte)
{
    LCD240_WriteData(WorkByte);
    LCD240_WriteCommand(0xc0);
}

/*-------------------------------------------------------------------
 LCD240_OutAscii
-------------------------------------------------------------------*/
void LCD240_OutAscii(unsigned char WorkByte)
{
    LCD240_WriteData(WorkByte-32);
    LCD240_WriteCommand(0xc0);
}

/*-------------------------------------------------------------------
 LCD240_OutString
-------------------------------------------------------------------*/
void LCD240_OutString(unsigned char WorkStr[])
{

unsigned int WorkCounter;

    WorkCounter = 0;

    while (WorkStr[WorkCounter] != 0)
    {
        LCD240_OutAscii(WorkStr[WorkCounter]);
    }
}

/*-------------------------------------------------------------------
 LCD240_OutStringXY
-------------------------------------------------------------------*/
void LCD240_OutStringXY(unsigned char WorkByteX,
                        unsigned char WorkByteY,
                        unsigned char WorkStr[])
{

unsigned int WorkCounter;

    LCD240_SetCursor (WorkByteX, WorkByteY);
    WorkCounter = 0;

    while (WorkStr[WorkCounter] != 0x00)
    {
        LCD240_OutAscii(WorkStr[WorkCounter]);
        WorkCounter=WorkCounter+1;
    }
}

/*-------------------------------------------------------------------
 LCD240_OutVal    r16 (ASCII)
-------------------------------------------------------------------*/
void LCD240_OutVal(unsigned char WorkByte)
{

unsigned char WorkByte2 = 0;

    WorkByte2 = (WorkByte & 0xf0) >> 4; // obere 4 Bits interessant
    if (WorkByte2 >= 10)                // Wert > 10?
    {
        WorkByte2 = WorkByte2 + 7;      // Dann 55 dazuzhlen sonst
    }
    WorkByte2 = WorkByte2 + 48;         // nur 48.
    LCD240_OutAscii (WorkByte2);

    WorkByte2 = (WorkByte & 0x0f);      // untere 4 Bits interessant
    if (WorkByte2 >= 10)                // Wert > 10?
    {
        WorkByte2 = WorkByte2 + 7;      // Dann 55 dazuzhlen sonst
    }
    WorkByte2 = WorkByte2 + 48;         // nur 48.
    LCD240_OutAscii (WorkByte2);
}

/*-------------------------------------------------------------------
 LCD240_OutVal16Bit    r16 (ASCII)
-------------------------------------------------------------------*/
void LCD240_OutVal16Bit(unsigned int WorkWord)
{

int WorkByte1 = 0;               // int weil wird negativ
unsigned char WorkByte2 = 0;

    for (WorkByte1=12; WorkByte1>=0; WorkByte1=WorkByte1 - 4)
    {                                       
        WorkByte2 = (WorkWord & (0x000f << WorkByte1)) >> WorkByte1;
        if (WorkByte2 >= 10)            // Wert > 10?
        {
            WorkByte2 = WorkByte2 + 7;  // Dann 55 dazuzhlen sonst
        }
        WorkByte2 = WorkByte2 + 48;     // nur 48.
        LCD240_OutAscii (WorkByte2);
    }
}

/*-------------------------------------------------------------------
 LCD240_OutVal32Bit    r16 (ASCII)
-------------------------------------------------------------------*/
void LCD240_OutVal32Bit(long unsigned int WorkLongWord)
{

    LCD240_OutVal16Bit(((WorkLongWord & 0xffff0000) >> 16));
    LCD240_OutVal16Bit((WorkLongWord & 0x0000ffff));

}

/*-------------------------------------------------------------------
 LCD240_PutImage
-------------------------------------------------------------------*/
void LCD240_PutImage(unsigned int XPos,
                     unsigned int YPos,
                     unsigned int ButtonNumber,
                     unsigned char Inverse)
{

unsigned char PictureWidth = 0x00;
unsigned char PictureHeight = 0x00;
unsigned int WorkCounterX = 0x0000;
unsigned int WorkCounterY = 0x0000;
unsigned int Index = 0x0000;
unsigned char WorkByte = 0x00;

    PictureWidth = PRG_RDB(&ButtonTable[ButtonNumber][0]);
    PictureHeight = PRG_RDB(&ButtonTable[ButtonNumber][1]);

    Index = 0x0002;

    for (WorkCounterY=0x0000; WorkCounterY<PictureHeight; WorkCounterY+=1)
    {
        LCD240_SetADP(VAR_GraphHomeAdr+XPos+((YPos+WorkCounterY)*40));

        for(WorkCounterX=0x0000; WorkCounterX<PictureWidth; WorkCounterX+=1)
        {
            WorkByte = PRG_RDB(&ButtonTable[ButtonNumber][Index]);
            if (Inverse != 0x00)
              WorkByte = ~WorkByte;

            LCD240_OutChar(WorkByte);
            Index+=1;
        }
    }
}

/*-------------------------------------------------------------------
 LCD240_InitLCD
-------------------------------------------------------------------*/
void LCD240_InitLCD(void)
{
    LCD240_CombineMode(0x01);
    LCD240_SetTextHomeAdr(0x0000);
    LCD240_TextArea(0x28);
    LCD240_SetGraphHomeAdr(0x0140);
    LCD240_GraphArea(0x28);
    LCD240_OfsRegSet(0x03);
    LCD240_CrsrPatSel(0x07);
    LCD240_SetCursor(0x05,0x05);
    LCD240_SetADP(0x0000);
    LCD240_ClearText();
    LCD240_ClearGraph();
    LCD240_DisplayMode(0x0c);
}

